import numpy as np
import pystan
import matplotlib.pyplot as plt

StanCode = """
    data {
        real<lower = 0.0> eta;
    }
    parameters {
        corr_matrix[3] corr;
    }
    model {
        corr ~ lkj_corr(eta);
    }
"""

eta = float(input('eta = '))

sm = pystan.StanModel(model_code = StanCode)

fit = sm.sampling(data = {'eta': eta}, iter = 100_000, n_jobs = 1)

print(fit)

def idx_r(r):
    return int((r + 1) * 5)

r_smpls = fit['corr']
cnt_r = np.zeros((10, 10, 10))
for v in r_smpls:
    cnt_r[idx_r(v[0,1])][idx_r(v[0,2])][idx_r(v[1,2])] += 1
    
cnt_max = cnt_r.max()
print(cnt_max)

def r_from_idx(i):
    return 2 * (i / 10) - 0.9

Low_cnt = []
Med_cnt = []
High_cnt = []
for i12 in range(10):
    for i13 in range(10):
        for i23 in range(10):
            if cnt_r[i12][i13][i23] > cnt_max/2:
                High_cnt.append([r_from_idx(i12),
                                 r_from_idx(i13),
                                 r_from_idx(i23)])
            elif cnt_r[i12][i13][i23] > cnt_max/4:
                 Med_cnt.append([r_from_idx(i12),
                                 r_from_idx(i13),
                                 r_from_idx(i23)])
            elif cnt_r[i12][i13][i23] > 0:
                 Low_cnt.append([r_from_idx(i12),
                                 r_from_idx(i13),
                                 r_from_idx(i23)])
High_cnt = np.array(High_cnt)
Med_cnt = np.array(Med_cnt)
Low_cnt = np.array(Low_cnt)
                
fig = plt.figure(figsize = (7,7))
ax = fig.add_subplot(projection = '3d')

if len(High_cnt) > 0:
    ax.scatter(High_cnt.T[0], High_cnt.T[1], High_cnt.T[2],
           color = 'r', label = '>max/2')  
if len(Med_cnt) > 0:
    ax.scatter(Med_cnt.T[0], Med_cnt.T[1], Med_cnt.T[2],
           color = 'g', label = 'max/4 ~ max/2')  
if len(Low_cnt) > 0:
    ax.scatter(Low_cnt.T[0], Low_cnt.T[1], Low_cnt.T[2],
           color = 'b', alpha = 0.3, label = '<max/4')
ax.set_xlabel(r'$\rho_{12}$', fontsize = 14)
ax.set_ylabel(r'$\rho_{13}$', fontsize = 14)
ax.set_zlabel(r'$\rho_{23}$', fontsize = 14)
plt.title(r'$\eta = ${}'.format(eta), fontsize = 20)
plt.legend()
plt.show()
